@@ -28,7 +28,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "galias.h"
 #include "glib.h"
 #include "gprintfint.h"
 #include "gthreadinit.h"
@@ -48,6 +47,8 @@
 #error GNU libiconv not in use but included iconv.h is from libiconv
 #endif
 
+#include "galias.h"
+
 GQuark 
 g_convert_error_quark (void)
 {
@@ -398,14 +399,17 @@ open_converter (const gchar *to_codeset,
   G_UNLOCK (iconv_cache_lock);
   
   /* Something went wrong.  */
-  if (errno == EINVAL)
-    g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_NO_CONVERSION,
-		 _("Conversion from character set '%s' to '%s' is not supported"),
-		 from_codeset, to_codeset);
-  else
-    g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
-		 _("Could not open converter from '%s' to '%s'"),
-		 from_codeset, to_codeset);
+  if (error)
+    {
+      if (errno == EINVAL)
+	g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_NO_CONVERSION,
+		     _("Conversion from character set '%s' to '%s' is not supported"),
+		     from_codeset, to_codeset);
+      else
+	g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
+		     _("Could not open converter from '%s' to '%s'"),
+		     from_codeset, to_codeset);
+    }
   
   return cd;
 }
@@ -463,7 +467,14 @@ close_converter (GIConv converter)
 /**
  * g_convert:
  * @str:           the string to convert
- * @len:           the length of the string
+ * @len:           the length of the string, or -1 if the string is 
+ *                 nul-terminated<footnote id="nul-unsafe">
+                     <para>
+                       Note that some encodings may allow nul bytes to 
+                       occur inside strings. In that case, using -1 for 
+                       the @len parameter is unsafe.
+                     </para>
+                   </footnote>. 
  * @to_codeset:    name of character set into which to convert @str
  * @from_codeset:  character set of @str.
  * @bytes_read:    location to store the number of bytes in the
@@ -526,7 +537,8 @@ g_convert (const gchar *str,
 /**
  * g_convert_with_iconv:
  * @str:           the string to convert
- * @len:           the length of the string
+ * @len:           the length of the string, or -1 if the string is 
+ *                 nul-terminated<footnoteref linkend="nul-unsafe"/>. 
  * @converter:     conversion descriptor from g_iconv_open()
  * @bytes_read:    location to store the number of bytes in the
  *                 input string that were successfully converted, or %NULL.
@@ -601,14 +613,16 @@ g_convert_with_iconv (const gchar *str,
 	    goto again;
 	  }
 	case EILSEQ:
-	  g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_ILLEGAL_SEQUENCE,
-		       _("Invalid byte sequence in conversion input"));
+	  if (error)
+	    g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_ILLEGAL_SEQUENCE,
+			 _("Invalid byte sequence in conversion input"));
 	  have_error = TRUE;
 	  break;
 	default:
-	  g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
-		       _("Error during conversion: %s"),
-		       g_strerror (errno));
+	  if (error)
+	    g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
+			 _("Error during conversion: %s"),
+			 g_strerror (errno));
 	  have_error = TRUE;
 	  break;
 	}
@@ -624,8 +638,9 @@ g_convert_with_iconv (const gchar *str,
 	{
           if (!have_error)
             {
-              g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_PARTIAL_INPUT,
-                           _("Partial character sequence at end of input"));
+	      if (error)
+		g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_PARTIAL_INPUT,
+			     _("Partial character sequence at end of input"));
               have_error = TRUE;
             }
 	}
@@ -646,14 +661,15 @@ g_convert_with_iconv (const gchar *str,
 /**
  * g_convert_with_fallback:
  * @str:          the string to convert
- * @len:          the length of the string
+ * @len:          the length of the string, or -1 if the string is 
+ *                nul-terminated<footnoteref linkend="nul-unsafe"/>. 
  * @to_codeset:   name of character set into which to convert @str
  * @from_codeset: character set of @str.
  * @fallback:     UTF-8 string to use in place of character not
- *                present in the target encoding. (This must be
- *                in the target encoding), if %NULL, characters
- *                not in the target encoding will be represented
- *                as Unicode escapes \uxxxx or \Uxxxxyyyy.
+ *                present in the target encoding. (The string must be
+ *                representable in the target encoding). 
+                  If %NULL, characters not in the target encoding will 
+                  be represented as Unicode escapes \uxxxx or \Uxxxxyyyy.
  * @bytes_read:   location to store the number of bytes in the
  *                input string that were successfully converted, or %NULL.
  *                Even if the conversion was successful, this may be 
@@ -916,7 +932,7 @@ strdup_len (const gchar *string,
  * @opsysstring:   a string in the encoding of the current locale. On Windows
  *                 this means the system codepage.
  * @len:           the length of the string, or -1 if the string is
- *                 nul-terminated.
+ *                 nul-terminated<footnoteref linkend="nul-unsafe"/>. 
  * @bytes_read:    location to store the number of bytes in the
  *                 input string that were successfully converted, or %NULL.
  *                 Even if the conversion was successful, this may be 
@@ -956,7 +972,7 @@ g_locale_to_utf8 (const gchar  *opsysstring,
  * g_locale_from_utf8:
  * @utf8string:    a UTF-8 encoded string 
  * @len:           the length of the string, or -1 if the string is
- *                 nul-terminated.
+ *                 nul-terminated<footnoteref linkend="nul-unsafe"/>. 
  * @bytes_read:    location to store the number of bytes in the
  *                 input string that were successfully converted, or %NULL.
  *                 Even if the conversion was successful, this may be 
@@ -1070,7 +1086,7 @@ g_get_filename_charsets (G_CONST_RETURN gchar ***filename_charsets)
       cache->charset = g_strdup (charset);
       
       p = getenv ("G_FILENAME_ENCODING");
-      if (p != NULL) 
+      if (p != NULL && p[0] != '\0') 
 	{
 	  cache->filename_charsets = g_strsplit (p, ",", 0);
 	  cache->is_utf8 = (strcmp (cache->filename_charsets[0], "UTF-8") == 0);
@@ -1112,7 +1128,7 @@ g_get_filename_charsets (G_CONST_RETURN gchar ***filename_charsets)
 gboolean
 g_get_filename_charsets (G_CONST_RETURN gchar ***filename_charsets) 
 {
-  static gchar *charsets[] = {
+  static const gchar *charsets[] = {
     "UTF-8",
     NULL
   };
@@ -1166,7 +1182,7 @@ _g_convert_thread_init (void)
  * g_filename_to_utf8:
  * @opsysstring:   a string in the encoding for filenames
  * @len:           the length of the string, or -1 if the string is
- *                 nul-terminated.
+ *                 nul-terminated<footnoteref linkend="nul-unsafe"/>. 
  * @bytes_read:    location to store the number of bytes in the
  *                 input string that were successfully converted, or %NULL.
  *                 Even if the conversion was successful, this may be 
@@ -1180,8 +1196,9 @@ _g_convert_thread_init (void)
  * @error:         location to store the error occuring, or %NULL to ignore
  *                 errors. Any of the errors in #GConvertError may occur.
  * 
- * Converts a string which is in the encoding used by GLib for filenames
- * into a UTF-8 string.
+ * Converts a string which is in the encoding used by GLib for
+ * filenames into a UTF-8 string. Note that on Windows GLib uses UTF-8
+ * for filenames.
  * 
  * Return value: The converted string, or %NULL on an error.
  **/
@@ -1243,7 +1260,8 @@ g_filename_to_utf8 (const gchar *opsysstring,
  * @error:         location to store the error occuring, or %NULL to ignore
  *                 errors. Any of the errors in #GConvertError may occur.
  * 
- * Converts a string from UTF-8 to the encoding used for filenames.
+ * Converts a string from UTF-8 to the encoding GLib uses for
+ * filenames. Note that on Windows GLib uses UTF-8 for filenames.
  * 
  * Return value: The converted string, or %NULL on an error.
  **/
@@ -2003,3 +2021,5 @@ g_filename_display_name (const gchar *filename)
   return display_name;
 }
 
+#define __G_CONVERT_C__
+#include "galiasdef.c"
