@@ -19,7 +19,8 @@
  */
 
 #include "config.h"
-#include "galias.h"
+
+#define G_STDIO_NO_WRAP_ON_UNIX
 
 #include "glib.h"
 
@@ -40,14 +41,155 @@
 
 #include "gstdio.h"
 
+#include "galias.h"
+
 #if !defined (G_OS_UNIX) && !defined (G_OS_WIN32)
 #error Please port this to your operating system
 #endif
 
 
+/**
+ * g_access:
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
+ * @mode: as in access()
+ *
+ * A wrapper for the POSIX access() function. This function is used to
+ * test a pathname for one or several of read, write or execute
+ * permissions, or just existence. On Windows, the underlying access()
+ * function in the C library only checks the READONLY attribute, and
+ * does not look at the ACL at all. Software that needs to handle file
+ * permissions on Windows more exactly should use the Win32 API.
+ *
+ * See the C library manual for more details about access().
+ *
+ * Returns: zero if the pathname refers to an existing file system
+ * object that has all the tested permissions, or -1 otherwise or on
+ * error.
+ * 
+ * Since: 2.8
+ */
+int
+g_access (const gchar *filename,
+	  int          mode)
+{
+#ifdef G_OS_WIN32
+  if (G_WIN32_HAVE_WIDECHAR_API ())
+    {
+      wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
+      int retval;
+      int save_errno;
+      
+      if (wfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = _waccess (wfilename, mode);
+      save_errno = errno;
+
+      g_free (wfilename);
+
+      errno = save_errno;
+      return retval;
+    }
+  else
+    {    
+      gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
+      int retval;
+      int save_errno;
+
+      if (cp_filename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = access (cp_filename, mode);
+      save_errno = errno;
+
+      g_free (cp_filename);
+
+      errno = save_errno;
+      return retval;
+    }
+#else
+  return access (filename, mode);
+#endif
+}
+
+/**
+ * g_chmod:
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
+ * @mode: as in chmod()
+ *
+ * A wrapper for the POSIX chmod() function. The chmod() function is
+ * used to set the permissions of a file system object. Note that on
+ * Windows the file protection mechanism is not at all POSIX-like, and
+ * the underlying chmod() function in the C library just sets or
+ * clears the READONLY attribute. It does not touch any ACL. Software
+ * that needs to manage file permissions on Windows exactly should
+ * use the Win32 API.
+ *
+ * See the C library manual for more details about chmod().
+ *
+ * Returns: zero if the operation succeeded, -1 on error.
+ * 
+ * Since: 2.8
+ */
+int
+g_chmod (const gchar *filename,
+	 int          mode)
+{
+#ifdef G_OS_WIN32
+  if (G_WIN32_HAVE_WIDECHAR_API ())
+    {
+      wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
+      int retval;
+      int save_errno;
+      
+      if (wfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = _wchmod (wfilename, mode);
+      save_errno = errno;
+
+      g_free (wfilename);
+
+      errno = save_errno;
+      return retval;
+    }
+  else
+    {    
+      gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
+      int retval;
+      int save_errno;
+
+      if (cp_filename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = chmod (cp_filename, mode);
+      save_errno = errno;
+
+      g_free (cp_filename);
+
+      errno = save_errno;
+      return retval;
+    }
+#else
+  return chmod (filename, mode);
+#endif
+}
+
 /**
  * g_open:
- * @filename: a pathname in the GLib file name encoding
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
  * @flags: as in open()
  * @mode: as in open()
  *
@@ -74,9 +216,18 @@ g_open (const gchar *filename,
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
       wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-      int retval = _wopen (wfilename, flags, mode);
-      int save_errno = errno;
+      int retval;
+      int save_errno;
       
+      if (wfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = _wopen (wfilename, flags, mode);
+      save_errno = errno;
+
       g_free (wfilename);
 
       errno = save_errno;
@@ -85,8 +236,17 @@ g_open (const gchar *filename,
   else
     {    
       gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
-      int retval = open (cp_filename, flags, mode);
-      int save_errno = errno;
+      int retval;
+      int save_errno;
+
+      if (cp_filename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = open (cp_filename, flags, mode);
+      save_errno = errno;
 
       g_free (cp_filename);
 
@@ -98,9 +258,78 @@ g_open (const gchar *filename,
 #endif
 }
 
+/**
+ * g_creat:
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
+ * @mode: as in creat()
+ *
+ * A wrapper for the POSIX creat() function. The creat() function is
+ * used to convert a pathname into a file descriptor, creating a file
+ * if necessar. Note that on POSIX systems file descriptors are
+ * implemented by the operating system. On Windows, it's the C library
+ * that implements creat() and file descriptors. The actual Windows
+ * API for opening files is something different.
+ *
+ * See the C library manual for more details about creat().
+ *
+ * Returns: a new file descriptor, or -1 if an error occurred. The
+ * return value can be used exactly like the return value from creat().
+ * 
+ * Since: 2.8
+ */
+int
+g_creat (const gchar *filename,
+	 int          mode)
+{
+#ifdef G_OS_WIN32
+  if (G_WIN32_HAVE_WIDECHAR_API ())
+    {
+      wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
+      int retval;
+      int save_errno;
+      
+      if (wfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = _wcreat (wfilename, mode);
+      save_errno = errno;
+
+      g_free (wfilename);
+
+      errno = save_errno;
+      return retval;
+    }
+  else
+    {    
+      gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
+      int retval;
+      int save_errno;
+
+      if (cp_filename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = creat (cp_filename, mode);
+      save_errno = errno;
+
+      g_free (cp_filename);
+
+      errno = save_errno;
+      return retval;
+    }
+#else
+  return creat (filename, mode);
+#endif
+}
+
 /**
  * g_rename:
- * @oldfilename: a pathname in the GLib file name encoding
+ * @oldfilename: a pathname in the GLib file name encoding (UTF-8 on Windows)
  * @newfilename: a pathname in the GLib file name encoding
  *
  * A wrapper for the POSIX rename() function. The rename() function 
@@ -124,10 +353,28 @@ g_rename (const gchar *oldfilename,
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
       wchar_t *woldfilename = g_utf8_to_utf16 (oldfilename, -1, NULL, NULL, NULL);
-      wchar_t *wnewfilename = g_utf8_to_utf16 (newfilename, -1, NULL, NULL, NULL);
-      int retval = _wrename (woldfilename, wnewfilename);
-      int save_errno = errno;
-      
+      wchar_t *wnewfilename;
+      int retval;
+      int save_errno;
+
+      if (woldfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      wnewfilename = g_utf8_to_utf16 (newfilename, -1, NULL, NULL, NULL);
+
+      if (wnewfilename == NULL)
+	{
+	  g_free (woldfilename);
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = _wrename (woldfilename, wnewfilename);
+      save_errno = errno;
+
       g_free (woldfilename);
       g_free (wnewfilename);
       
@@ -137,9 +384,27 @@ g_rename (const gchar *oldfilename,
   else
     {
       gchar *cp_oldfilename = g_locale_from_utf8 (oldfilename, -1, NULL, NULL, NULL);
-      gchar *cp_newfilename = g_locale_from_utf8 (newfilename, -1, NULL, NULL, NULL);
-      int retval = rename (cp_oldfilename, cp_newfilename);
-      int save_errno = errno;
+      gchar *cp_newfilename;
+      int retval;
+      int save_errno;
+
+      if (cp_oldfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      cp_newfilename = g_locale_from_utf8 (newfilename, -1, NULL, NULL, NULL);
+
+      if (cp_newfilename == NULL)
+	{
+	  g_free (cp_oldfilename);
+	  errno = EINVAL;
+	  return -1;
+	}
+	
+      retval = rename (cp_oldfilename, cp_newfilename);
+      save_errno = errno;
 
       g_free (cp_oldfilename);
       g_free (cp_newfilename);
@@ -154,7 +419,7 @@ g_rename (const gchar *oldfilename,
 
 /**
  * g_mkdir: 
- * @filename: a pathname in the GLib file name encoding
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
  * @mode: permissions to use for the newly created directory
  *
  * A wrapper for the POSIX mkdir() function. The mkdir() function 
@@ -175,8 +440,17 @@ g_mkdir (const gchar *filename,
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
       wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-      int retval = _wmkdir (wfilename);
-      int save_errno = errno;
+      int retval;
+      int save_errno;
+
+      if (wfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = _wmkdir (wfilename);
+      save_errno = errno;
 
       g_free (wfilename);
       
@@ -186,8 +460,17 @@ g_mkdir (const gchar *filename,
   else
     {
       gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
-      int retval = mkdir (cp_filename);
-      int save_errno = errno;
+      int retval;
+      int save_errno;
+
+      if (cp_filename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = mkdir (cp_filename);
+      save_errno = errno;
 
       g_free (cp_filename);
 
@@ -199,9 +482,71 @@ g_mkdir (const gchar *filename,
 #endif
 }
 
+/**
+ * g_chdir: 
+ * @path: a pathname in the GLib file name encoding (UTF-8 on Windows)
+ *
+ * A wrapper for the POSIX chdir() function. The function changes the
+ * current directory of the process to @path.
+ * 
+ * See your C library manual for more details about chdir().
+ *
+ * Returns: 0 on success, -1 if an error occurred.
+ * 
+ * Since: 2.8
+ */
+int
+g_chdir (const gchar *path)
+{
+#ifdef G_OS_WIN32
+  if (G_WIN32_HAVE_WIDECHAR_API ())
+    {
+      wchar_t *wpath = g_utf8_to_utf16 (path, -1, NULL, NULL, NULL);
+      int retval;
+      int save_errno;
+
+      if (wpath == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = _wchdir (wpath);
+      save_errno = errno;
+
+      g_free (wpath);
+      
+      errno = save_errno;
+      return retval;
+    }
+  else
+    {
+      gchar *cp_path = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
+      int retval;
+      int save_errno;
+
+      if (cp_path == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = chdir (cp_path);
+      save_errno = errno;
+
+      g_free (cp_path);
+
+      errno = save_errno;
+      return retval;
+    }
+#else
+  return chdir (path);
+#endif
+}
+
 /**
  * g_stat: 
- * @filename: a pathname in the GLib file name encoding
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
  * @buf: a pointer to a <structname>stat</structname> struct, which
  *    will be filled with the file information
  *
@@ -223,8 +568,17 @@ g_stat (const gchar *filename,
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
       wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-      int retval = _wstat (wfilename, (struct _stat *) buf);
-      int save_errno = errno;
+      int retval;
+      int save_errno;
+
+      if (wfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = _wstat (wfilename, (struct _stat *) buf);
+      save_errno = errno;
 
       g_free (wfilename);
 
@@ -234,8 +588,17 @@ g_stat (const gchar *filename,
   else
     {
       gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
-      int retval = stat (cp_filename, buf);
-      int save_errno = errno;
+      int retval;
+      int save_errno;
+
+      if (cp_filename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = stat (cp_filename, buf);
+      save_errno = errno;
 
       g_free (cp_filename);
 
@@ -249,7 +612,7 @@ g_stat (const gchar *filename,
 
 /**
  * g_lstat: 
- * @filename: a pathname in the GLib file name encoding
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
  * @buf: a pointer to a <structname>stat</structname> struct, which
  *    will be filled with the file information
  *
@@ -280,7 +643,7 @@ g_lstat (const gchar *filename,
 
 /**
  * g_unlink:
- * @filename: a pathname in the GLib file name encoding
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
  *
  * A wrapper for the POSIX unlink() function. The unlink() function 
  * deletes a name from the filesystem. If this was the last link to the 
@@ -303,8 +666,17 @@ g_unlink (const gchar *filename)
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
       wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-      int retval = _wunlink (wfilename);
-      int save_errno = errno;
+      int retval;
+      int save_errno;
+
+      if (wfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = _wunlink (wfilename);
+      save_errno = errno;
 
       g_free (wfilename);
 
@@ -314,8 +686,17 @@ g_unlink (const gchar *filename)
   else
     {
       gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
-      int retval = unlink (cp_filename);
-      int save_errno = errno;
+      int retval;
+      int save_errno;
+
+      if (cp_filename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = unlink (cp_filename);
+      save_errno = errno;
 
       g_free (cp_filename);
 
@@ -329,7 +710,7 @@ g_unlink (const gchar *filename)
 
 /**
  * g_remove:
- * @filename: a pathname in the GLib file name encoding
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
  *
  * A wrapper for the POSIX remove() function. The remove() function
  * deletes a name from the filesystem.
@@ -358,6 +739,12 @@ g_remove (const gchar *filename)
       int retval;
       int save_errno;
 
+      if (wfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
       retval = _wremove (wfilename);
       if (retval == -1)
 	retval = _wrmdir (wfilename);
@@ -374,6 +761,12 @@ g_remove (const gchar *filename)
       int retval;
       int save_errno;
       
+      if (cp_filename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
       retval = remove (cp_filename);
       if (retval == -1)
 	retval = rmdir (cp_filename);
@@ -391,7 +784,7 @@ g_remove (const gchar *filename)
 
 /**
  * g_rmdir:
- * @filename: a pathname in the GLib file name encoding
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
  *
  * A wrapper for the POSIX rmdir() function. The rmdir() function
  * deletes a directory from the filesystem.
@@ -411,8 +804,17 @@ g_rmdir (const gchar *filename)
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
       wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-      int retval = _wrmdir (wfilename);
-      int save_errno = errno;
+      int retval;
+      int save_errno;
+
+      if (wfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+      
+      retval = _wrmdir (wfilename);
+      save_errno = errno;
 
       g_free (wfilename);
 
@@ -422,8 +824,17 @@ g_rmdir (const gchar *filename)
   else
     {
       gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
-      int retval = rmdir (cp_filename);
-      int save_errno = errno;
+      int retval;
+      int save_errno;
+
+      if (cp_filename == NULL)
+	{
+	  errno = EINVAL;
+	  return -1;
+	}
+
+      retval = rmdir (cp_filename);
+      save_errno = errno;
 
       g_free (cp_filename);
 
@@ -437,7 +848,7 @@ g_rmdir (const gchar *filename)
 
 /**
  * g_fopen:
- * @filename: a pathname in the GLib file name encoding
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
  * @mode: a string describing the mode in which the file should be 
  *   opened
  *
@@ -459,9 +870,27 @@ g_fopen (const gchar *filename,
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
       wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-      wchar_t *wmode = g_utf8_to_utf16 (mode, -1, NULL, NULL, NULL);
-      FILE *retval = _wfopen (wfilename, wmode);
-      int save_errno = errno;
+      wchar_t *wmode;
+      FILE *retval;
+      int save_errno;
+
+      if (wfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return NULL;
+	}
+
+      wmode = g_utf8_to_utf16 (mode, -1, NULL, NULL, NULL);
+
+      if (wmode == NULL)
+	{
+	  g_free (wfilename);
+	  errno = EINVAL;
+	  return NULL;
+	}
+	
+      retval = _wfopen (wfilename, wmode);
+      save_errno = errno;
 
       g_free (wfilename);
       g_free (wmode);
@@ -472,8 +901,17 @@ g_fopen (const gchar *filename,
   else
     {
       gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
-      FILE *retval = fopen (cp_filename, mode);
-      int save_errno = errno;
+      FILE *retval;
+      int save_errno;
+
+      if (cp_filename == NULL)
+	{
+	  errno = EINVAL;
+	  return NULL;
+	}
+
+      retval = fopen (cp_filename, mode);
+      save_errno = errno;
 
       g_free (cp_filename);
 
@@ -487,7 +925,7 @@ g_fopen (const gchar *filename,
 
 /**
  * g_freopen:
- * @filename: a pathname in the GLib file name encoding
+ * @filename: a pathname in the GLib file name encoding (UTF-8 on Windows)
  * @mode: a string describing the mode in which the file should be 
  *   opened
  * @stream: an existing stream which will be reused, or %NULL
@@ -511,9 +949,27 @@ g_freopen (const gchar *filename,
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
       wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-      wchar_t *wmode = g_utf8_to_utf16 (mode, -1, NULL, NULL, NULL);
-      FILE *retval = _wfreopen (wfilename, wmode, stream);
-      int save_errno = errno;
+      wchar_t *wmode;
+      FILE *retval;
+      int save_errno;
+
+      if (wfilename == NULL)
+	{
+	  errno = EINVAL;
+	  return NULL;
+	}
+      
+      wmode = g_utf8_to_utf16 (mode, -1, NULL, NULL, NULL);
+
+      if (wmode == NULL)
+	{
+	  g_free (wfilename);
+	  errno = EINVAL;
+	  return NULL;
+	}
+      
+      retval = _wfreopen (wfilename, wmode, stream);
+      save_errno = errno;
 
       g_free (wfilename);
       g_free (wmode);
@@ -524,8 +980,17 @@ g_freopen (const gchar *filename,
   else
     {
       gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
-      FILE *retval = freopen (cp_filename, mode, stream);
-      int save_errno = errno;
+      FILE *retval;
+      int save_errno;
+
+      if (cp_filename == NULL)
+	{
+	  errno = EINVAL;
+	  return NULL;
+	}
+
+      retval = freopen (cp_filename, mode, stream);
+      save_errno = errno;
 
       g_free (cp_filename);
 
@@ -536,3 +1001,6 @@ g_freopen (const gchar *filename,
   return freopen (filename, mode, stream);
 #endif
 }
+
+#define __G_STDIO_C__
+#include "galiasdef.c"
