@@ -28,6 +28,7 @@
 #include "glib.h"
 #include "gunichartables.h"
 #include "gunicodeprivate.h"
+#include "galias.h"
 
 #define ATTR_TABLE(Page) (((Page) <= G_UNICODE_LAST_PAGE_PART1) \
                           ? attr_table_part1[Page] \
@@ -377,7 +378,7 @@ g_unichar_toupper (gunichar c)
       gunichar val = ATTTABLE (c >> 8, c & 0xff);
       if (val >= 0x1000000)
 	{
-	  const guchar *p = special_case_table + val - 0x1000000;
+	  const gchar *p = special_case_table + val - 0x1000000;
 	  return g_utf8_get_char (p);
 	}
       else
@@ -414,7 +415,7 @@ g_unichar_tolower (gunichar c)
       gunichar val = ATTTABLE (c >> 8, c & 0xff);
       if (val >= 0x1000000)
 	{
-	  const guchar *p = special_case_table + val - 0x1000000;
+	  const gchar *p = special_case_table + val - 0x1000000;
 	  return g_utf8_get_char (p);
 	}
       else
@@ -524,7 +525,16 @@ typedef enum {
 static LocaleType
 get_locale_type (void)
 {
+#ifdef G_OS_WIN32
+  char *tem = g_win32_getlocale ();
+  char locale[2];
+
+  locale[0] = tem[0];
+  locale[1] = tem[1];
+  g_free (tem);
+#else
   const char *locale = setlocale (LC_CTYPE, NULL);
+#endif
 
   switch (locale[0])
     {
@@ -578,7 +588,7 @@ output_special_case (gchar *out_buffer,
 		     int    type,
 		     int    which)
 {
-  const guchar *p = special_case_table + offset;
+  const gchar *p = special_case_table + offset;
   gint len;
 
   if (type != G_UNICODE_TITLECASE_LETTER)
@@ -942,11 +952,12 @@ gchar *
 g_utf8_casefold (const gchar *str,
 		 gssize       len)
 {
-  GString *result = g_string_new (NULL);
+  GString *result;
   const char *p;
 
   g_return_val_if_fail (str != NULL, NULL);
 
+  result = g_string_new (NULL);
   p = str;
   while ((len < 0 || p < str + len) && *p)
     {
@@ -1044,3 +1055,6 @@ g_unichar_get_mirror_char (gunichar ch,
   return found;
 
 }
+
+#define __G_UNIPROP_C__
+#include "galiasdef.c"
